<?php

namespace App\Console\Commands;

use App\Mail\CronsReport;
use App\Repositories\CronsRepository;
use App\Traits\TranslateCommandExitCode;
use Carbon\Carbon;
use Mail;

class WaybillCreateCron extends BaseCommand
{
    use TranslateCommandExitCode;

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'waybill:create:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Выполнение заданий на генерацию документов форм 1, 4с и ЭСМ 7 и отправка полученных документов по почте';

    /**
     * Execute the console command.
     *
     * @param CronsRepository $cronsRepository
     *
     * @return mixed
     */
    public function handle(CronsRepository $cronsRepository)
    {
        $tasks = $cronsRepository->findWhereIn('type', ['4s', 'f1', 'esm7'])->groupBy('users_id');

        $start_date = Carbon::now();
        $end_date = clone $start_date;

        foreach ($tasks as $task) {
            foreach ($task as $item) {
                $files = [];

                foreach ($item->objects as $object) {
                    $waybill = $this->waybillsRepo->create([
                        'type'       => $item->type,
                        'start_date' => $start_date->subDay(),
                        'end_date'   => $end_date,
                        'owners_id'  => $item->owners_id,
                        'drivers_id' => $item->drivers_id,
                        'trailer'    => $item->trailer,
                        'customer'   => $item->customer,
                        'place_work' => $item->place_work,
                        'type_work'  => $item->type_work,
                        'objects_id' => $object->objects_id,
                        'users_id'   => $item->users_id,
                    ]);

                    if (!$waybill) {
                        continue;
                    }

                    $exitCode = \Artisan::call('waybill:create', ['id' => $waybill->id, 'type' => $waybill->type]);

                    if ($this->getValueExitCode(BaseCommand::class, $exitCode) === 'SUCCESS') {
                        $waybill = $this->waybillsRepo->find($waybill->id);

                        $files[] = $waybill->file;
                    }
                }

                Mail::to($item->email)->send(new CronsReport($files));
                unset($files);
            }
        }

        return $this;
    }
}
