<?php

namespace App\Console\Commands;

use App\Sdk\SdkManager;

class WaybillCreateesm4 extends BaseCommand
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'waybill:create:esm4 {id}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Создание документа ЭСМ - 4';


    /**
     * Execute the console command.
     *
     * @return mixed
     * @throws \PHPExcel_Reader_Exception
     */
    public function handle()
    {
        $waybill = $this->waybillsRepo->findWithoutFail($this->argument('id'));

        if (!$waybill) {
            $this->error('Не найден путевой лист');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $start_date = $waybill->start_date;
        $end_date = $waybill->end_date;

        $sdk = (new SdkManager())->instance($waybill->user->server);

        if ($sdk === null) {
            $this->error('Не найден SDK сервера');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $sdk->setBaseApiUrl();

        if (!$response = $sdk->login(['username' => $waybill->user->username, 'password' => decrypt($waybill->user->danger_password)])) {
            $this->error('Ошибка авторизации SDK сервера');

            return self::$EXIT_CODE['FAILED'];
        }

        $reportTrip = $sdk->getReport($response->user->gd);

        if (count($reportTrip) === 0) {
            $this->error('Не найден шаблон отчета');

            return self::$EXIT_CODE['FAILED'];
        }

        $unit = $sdk->getUnit($waybill->object->mon_id);

        if ($unit['data']['id'] === 0) {
            $this->error('Объект не найден на сервере');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $organisation = '';
        $gos_num = $waybill->object->gos_number;
        $auto_mark = $waybill->object->model;
        $fuel_mark = $waybill->object->fuel;

        if (count($unit['data']['fields']) > 0) {
            foreach ($unit['data']['fields'] as $custom_field) {
                $custom_field_name = $custom_field->nm;
                $custom_field_value = $custom_field->vl;

                if (empty($gos_num) && $custom_field_name === 'Государственный номерной знак') {
                    $gos_num = $custom_field_value;
                } elseif (empty($auto_mark) && $custom_field_name === 'Марка автомобиля') {
                    $auto_mark = $custom_field_value;
                } elseif (empty($organisation) && $custom_field_name === 'Организация') {
                    $organisation = $custom_field_value;
                }
            }
        }

        if (isset($waybill->owner) && !empty($waybill->owner)) {
            if ($waybill->owner->type === 'company') {
                $organisation = implode(' ', [$waybill->owner->company, ',', $waybill->owner->address, ',', $waybill->owner->phone]);
            } else {
                $organisation = implode(' ', [$waybill->owner->surname, $waybill->owner->name, $waybill->owner->middlename, ',', $waybill->owner->address, ',', $waybill->owner->phone]);
            }
        }

        $execReport = $sdk->execReport($reportTrip, $waybill->object->mon_id, $start_date, $end_date);

        if (!$execReport) {
            $this->error('Ошибка выполнения отчета');

            return self::$EXIT_CODE['FAILED'];
        }

        $stats = $sdk->getStatsReport();
        $tables = $sdk->getTablesReport();

        if (count($tables) === 0) {
            $this->error('Данные не получены');

            return self::$EXIT_CODE['FAILED'];
        }

        $spent_fact = $spent_norm = '';

        foreach ($stats as $stat) {
            list($stat_name, $stat_value) = (array)$stat;

            if ($stat_name === 'Потрачено по нормам в поездках') {
                $spent_norm = (float)$stat_value;
            } elseif ($stat_name === 'Потрачено по расчету в поездках') {
                $spent_fact = (float)$stat_value;
            }
        }

        try {
            $objReader = \PHPExcel_IOFactory::createReader('Excel5');
            $objPHPExcel = $objReader->load(resource_path('/waytemplate/esm4.xls'));
            $objPHPExcel->setActiveSheetIndex();
            $aSheet = $objPHPExcel->getActiveSheet();
        } catch (\PHPExcel_Writer_Exception $e) {
            $this->error('Ошибка открытия excel шаблона');

            return self::$EXIT_CODE['FAILED'];
        } catch (\PHPExcel_Exception $e) {
            $this->error('Ошибка excel шаблона');

            return self::$EXIT_CODE['FAILED'];
        }

        $aSheet->setCellValue('BQ18', $end_date->format('d.m.Y'));
        $aSheet->setCellValue('CS18', $start_date->format('d.m.Y'));
        $aSheet->setCellValue('CZ18', $end_date->format('d.m.Y'));
        $aSheet->setCellValue('DU18', $auto_mark);

        if (!empty($organisation)) {
            $aSheet->setCellValue('Q6', $organisation);
        }

        if ($waybill->customer) {
            $aSheet->setCellValue('M8', $waybill->customer);
        }

        if ($waybill->place_work) {
            $aSheet->setCellValue('K10', $waybill->place_work);
        }

        $aSheet->setCellValue('L16', $waybill->object->name);

        if (stripos($fuel_mark, 'аи') !== false || stripos($fuel_mark, 'бен') !== false) {
            $aSheet->setCellValue('N82', $spent_fact);
            $aSheet->setCellValue('N83', $spent_norm);
        } elseif (stripos($fuel_mark, 'диз') !== false || stripos($fuel_mark, 'дт') !== false) {
            $aSheet->setCellValue('W82', $spent_fact);
            $aSheet->setCellValue('W83', $spent_norm);
        } else {
            $aSheet->setCellValue('AH82', $spent_fact);
            $aSheet->setCellValue('AH83', $spent_norm);
        }

        $driver_name = '';

        foreach ($tables as $table_index => $table) {
            if ($table->nm === 'unit_trips') {
                $headers = (array)$table->h;
                $headers_assoc = [];

                $header_num = 0;
                foreach ($headers as $header) {
                    $headers_assoc[$header] = $header_num;

                    $header_num++;
                }

                $params = [
                    'tableIndex' => $table_index,
                    'indexFrom'  => 0,
                    'indexTo'    => $table->r,
                ];

                $response = $sdk->call('report/get_result_rows', $params);
                $trips = $response->body;

                $trip_num = 1;
                foreach ($trips as $trip) {
                    $ctrip_data_arr = (array)$trip->c;

                    foreach ($ctrip_data_arr as $data_num => $ctrip_data) {
                        $value = is_object($ctrip_data) ? $ctrip_data->{'t'} : $ctrip_data;

                        if ($trip_num === count($trips)) {
                            if ($data_num === $headers_assoc['Водитель']) {
                                $driver_name = $value;
                            }
                        }

                        if ($data_num === $headers_assoc['Водитель']) {
                            continue;
                        }
                    }

                    $trip_num++;
                }
            }
        }

        if (isset($waybill->driver) && !empty($waybill->driver)) {
            $driver_name = implode(' ', [$waybill->driver->surname, $waybill->driver->name, $waybill->driver->middlename]);
        }

        $aSheet->setCellValue('A24', $driver_name);

        $row = 38;
        $row2 = 67;
        for ($i = 0; $i <= $end_date->diffInDays($start_date); $i++) {
            $tmp_date = $start_date->copy()->addDays($i);

            $aSheet->setCellValue($this->markWorkDay($tmp_date->format('d')), 'X');

            $aSheet->setCellValue('A' . $row, $tmp_date->format('d'));
            $aSheet->setCellValue('EO' . $row, $tmp_date->format('d.m.Y'));
            $aSheet->setCellValue('A' . $row2, $tmp_date->format('d.m'));

            if ($waybill->type_work) {
                $aSheet->setCellValue('L' . $row, $waybill->type_work);
            }

            if ($row === 43) {
                $row = 50;
            }

            $execReport = $sdk->execReport($reportTrip, $waybill->object->mon_id, $start_date, $tmp_date);

            if (!$execReport) {
                continue;
            }

            $stats = $sdk->getStatsReport();

            if (count($stats) === 0) {
                continue;
            }

            foreach ($stats as $stat) {
                list($stat_name, $stat_value) = (array)$stat;

                if ($stat_name === 'Нач. уровень') {
                    $aSheet->setCellValue('G' . $row2, (float)$stat_value);
                } elseif ($stat_name === 'Всего заправлено') {
                    if (stripos($fuel_mark, 'аи') !== false || stripos($fuel_mark, 'бен') !== false) {
                        $aSheet->setCellValue('N' . $row2, (float)$stat_value);
                    } elseif (stripos($fuel_mark, 'диз') !== false || stripos($fuel_mark, 'дт') !== false) {
                        $aSheet->setCellValue('W' . $row2, (float)$stat_value);
                    } else {
                        $aSheet->setCellValue('AH' . $row2, (float)$stat_value);
                    }
                } elseif ($stat_name === 'Конеч. уровень') {
                    $aSheet->setCellValue('AO' . $row2, (float)$stat_value);
                }
            }

            $row++;
            $row2++;
        }

        $aSheet->setCellValue('CQ68', $start_date->format('d'));
        $aSheet->setCellValue('CW68', $start_date->format('m'));
        $aSheet->setCellValue('DO68', $start_date->format('Y'));

        $aSheet->setCellValue('BT71', $start_date->format('d'));
        $aSheet->setCellValue('BZ71', $start_date->format('m'));
        $aSheet->setCellValue('CR71', $start_date->format('Y'));

        $aSheet->setCellValue('BT74', $end_date->format('d'));
        $aSheet->setCellValue('BZ74', $end_date->format('m'));
        $aSheet->setCellValue('CR74', $end_date->format('Y'));

        try {
            $objWriter = \PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');

            $path = 'waybills/' . $waybill->object->name;

            if (!@mkdir(resource_path($path), 0700, true) && !is_dir(resource_path($path))) {
                $this->error('Ошибка создания папки для хранения excel документов');

                return self::$EXIT_CODE['FAILED'];
            }

            $file = $path . '/ЭСМ - 4 № ' . $waybill->id . '.xls';

            $objWriter->save(resource_path($file));

            $this->waybillsRepo->update(['file' => $file], $waybill->id);

            return self::$EXIT_CODE['SUCCESS'];
        } catch (\PHPExcel_Writer_Exception $e) {
            $this->error('Ошибка сохранения excel документа');

            return self::$EXIT_CODE['FAILED'];
        }
    }

    /**
     * @param int $day
     * @return string
     */
    protected function markWorkDay($day = 1)
    {
        $array = [
            '01' => 'AL24',
            '02' => 'AO24',
            '03' => 'AR24',
            '04' => 'AU24',
            '05' => 'AX24',
            '06' => 'BA24',
            '07' => 'BD24',
            '08' => 'BG24',
            '09' => 'BJ24',
            '10' => 'BM24',
            '11' => 'BP24',
            '12' => 'BS24',
            '13' => 'BV24',
            '14' => 'BY24',
            '15' => 'CB24',
            '16' => 'AL25',
            '17' => 'AO25',
            '18' => 'AR25',
            '19' => 'AU25',
            '20' => 'AX25',
            '21' => 'BA25',
            '22' => 'BD25',
            '23' => 'BG25',
            '24' => 'BJ25',
            '25' => 'BM25',
            '26' => 'BP25',
            '27' => 'BS25',
            '28' => 'BV25',
            '29' => 'BY25',
            '30' => 'CB25',
            '31' => 'CE25',
        ];

        return (string) $array[$day];
    }
}
