<?php

namespace App\DataTables;

use App\Models\Crons;
use Yajra\Datatables\Services\DataTable;

class CronsDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'crons.datatables_actions')
            ->editColumn('users_id', function (Crons $crons) {
                return $crons->user->username;
            })
            ->editColumn('active', function (Crons $crons) {
                if ($crons->active) {
                    return 'Да';
                }

                return 'Нет';
            })
            ->editColumn('type', function (Crons $crons) {
                return getTypesWaybill($crons->type);
            })
            ->editColumn('objects', function (Crons $crons) {
                $objects = [];
                foreach ($crons->objects as $object) {
                    $objects[] = @$object->object->name;
                }

                return implode(', ', $objects);
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $crons = Crons::query();

        if (!\Auth::user()->isAdmin()) {
            $crons = $crons->where('users_id', \Auth::user()->id);
        }

        return $this->applyScopes($crons);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '10%', 'title' => 'Управление'])
            ->ajax('')
            ->parameters([
                'dom'      => 'Bfrtip',
                'scrollX'  => false,
                'language' => [
                    'url' => url('vendor/datatables/Russian.json'),
                ],
                'buttons'  => [
                    'print',
                    'reset',
                    'reload',
                    [
                        'extend'  => 'collection',
                        'text'    => '<i class="fa fa-download"></i> Экспорт',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    'colvis',
                ],
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        $array = [];

        if (\Auth::user()->isAdmin()) {
            $array['users_id'] = ['name' => 'users_id', 'data' => 'users_id', 'title' => 'Пользователь'];
        }

        $array['active'] = ['name' => 'active', 'data' => 'active', 'title' => 'Включено'];
        $array['type'] = ['name' => 'type', 'data' => 'type', 'title' => 'Тип документа'];
        $array['email'] = ['name' => 'email', 'data' => 'email', 'title' => 'Email'];
        $array['objects'] = ['name' => 'objects', 'data' => 'objects', 'title' => 'Объекты'];

        return $array;
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'crons';
    }
}
