<?php

namespace App\DataTables;

use App\Models\Drivers;
use Form;
use Yajra\Datatables\Services\DataTable;

class DriversDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'drivers.datatables_actions')
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $drivers = Drivers::query();

        if (!\Auth::user()->isAdmin()) {
            $drivers = $drivers->where('users_id', \Auth::user()->id);
        }

        return $this->applyScopes($drivers);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '10%', 'title' => 'Управление'])
            ->ajax('')
            ->parameters([
                'dom'      => 'Bfrtip',
                'scrollX'  => false,
                'language' => [
                    'url' => url('vendor/datatables/Russian.json'),
                ],
                'buttons'  => [
                    'print',
                    'reset',
                    'reload',
                    [
                        'extend'  => 'collection',
                        'text'    => '<i class="fa fa-download"></i> Экспорт',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    'colvis',
                ],
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'surname'    => ['name' => 'surname', 'data' => 'surname', 'title' => 'Фамилия'],
            'name'       => ['name' => 'name', 'data' => 'name', 'title' => 'Имя'],
            'middlename' => ['name' => 'middlename', 'data' => 'middlename', 'title' => 'Отчество'],
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'drivers';
    }
}
