<?php

namespace App\DataTables;

use Auth;
use App\Models\Objects;
use Form;
use Yajra\Datatables\Services\DataTable;

class ObjectsDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'objects.datatables_actions')
            ->editColumn('users_id', function (Objects $object) {
                return $object->user->username;
            })
            ->editColumn('servers_id', function (Objects $object) {
                if (isset($object->servers_id) && !empty($object->servers_id)) {
                    return $object->server->name;
                }

                return $object->servers_id;
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $objects = Objects::query();

        if (!\Auth::user()->isAdmin()) {
            $objects = $objects->where('users_id', \Auth::user()->id);
        }

        return $this->applyScopes($objects);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '10%', 'title' => 'Управление'])
            ->ajax('')
            ->parameters([
                'dom'      => 'Bfrtip',
                'scrollX'  => false,
                'language' => [
                    'url' => url('vendor/datatables/Russian.json'),
                ],
                'buttons'  => [
                    'print',
                    'reset',
                    'reload',
                    [
                        'extend'  => 'collection',
                        'text'    => '<i class="fa fa-download"></i> Экспорт',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    'colvis',
                ],
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        $columns = [
            'name'       => ['name' => 'name', 'data' => 'name', 'title' => 'Имя'],
            'gos_number' => ['name' => 'gos_number', 'data' => 'gos_number', 'title' => 'Гос. номер'],
            'type'       => ['name' => 'type', 'data' => 'type', 'title' => 'Тип'],
            'mon_id'     => ['name' => 'mon_id', 'data' => 'mon_id', 'title' => 'Ид в мониторинге'],
        ];

        if (!Auth::user()->isAdmin()) {
            return $columns;
        }

        return array_merge($columns, [
            'users_id'   => ['name' => 'users_id', 'data' => 'users_id', 'title' => 'Пользователь'],
            'servers_id' => ['name' => 'servers_id', 'data' => 'servers_id', 'title' => 'Сервер'],
        ]);
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'objects';
    }
}
