<?php

namespace App\DataTables;

use App\Models\Users;
use Form;
use Yajra\Datatables\Services\DataTable;

class UsersDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'users.datatables_actions')
            ->editColumn('servers_id', function (Users $user) {
                if (isset($user->servers_id) && !empty($user->servers_id)) {
                    return $user->server->name;
                }

                return $user->servers_id;
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $users = Users::query();

        return $this->applyScopes($users);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '10%', 'title   ' => 'Управление'])
            ->ajax('')
            ->parameters([
                'dom'      => 'Bfrtip',
                'scrollX'  => false,
                'language' => [
                    'url' => url('vendor/datatables/Russian.json'),
                ],
                'buttons'  => [
                    'print',
                    'reset',
                    'reload',
                    [
                        'extend'  => 'collection',
                        'text'    => '<i class="fa fa-download"></i> Экспорт',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    'colvis',
                ],
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'username'   => ['name' => 'username', 'data' => 'username', 'title' => 'Логин'],
            'mon_id'     => ['name' => 'mon_id', 'data' => 'mon_id', 'title' => 'Ид в мониторинге'],
            'servers_id' => ['name' => 'servers_id', 'data' => 'servers_id', 'title' => 'Сервер'],
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'users';
    }
}
