<?php

namespace App\DataTables;

use App\Models\Waybills;
use Form;
use Yajra\Datatables\Services\DataTable;

class WaybillsDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'waybills.datatables_actions')
            ->editColumn('users_id', function (Waybills $waybill) {
                return $waybill->user->username;
            })
            ->editColumn('drivers_id', function (Waybills $waybill) {
                if (isset($waybill->drivers_id) && !empty($waybill->drivers_id)) {
                    return implode(' ', [$waybill->driver->surname, $waybill->driver->name, $waybill->driver->middlename]);
                }

                return $waybill->drivers_id;
            })
            ->editColumn('objects_id', function (Waybills $waybill) {
                if (isset($waybill->objects_id) && !empty($waybill->objects_id)) {
                    return $waybill->object->name;
                }

                return $waybill->objects_id;
            })
            ->editColumn('trailer', function (Waybills $waybill) {
                if (isset($waybill->trailer) && !empty($waybill->trailer)) {
                    return $waybill->trailerF->name;
                }

                return $waybill->trailer;
            })
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $waybills = Waybills::query();

        if (!\Auth::user()->isAdmin()) {
            $waybills = $waybills->where('users_id', \Auth::user()->id);
        }

        return $this->applyScopes($waybills);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '10%', 'title' => 'Управление'])
            ->ajax('')
            ->parameters([
                'dom'      => 'Bfrtip',
                'scrollX'  => false,
                'language' => [
                    'url' => url('vendor/datatables/Russian.json'),
                ],
                'buttons'  => [
                    'print',
                    'reset',
                    'reload',
                    [
                        'extend'  => 'collection',
                        'text'    => '<i class="fa fa-download"></i> Экспорт',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    'colvis',
                ],
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        $array = [
            'id'   => ['name' => 'id', 'data' => 'id', 'title' => 'ИД'],
            'start_date'   => ['name' => 'start_date', 'data' => 'start_date', 'title' => 'Начальная дата'],
            'end_date'     => ['name' => 'end_date', 'data' => 'end_date', 'title' => 'Конечная дата'],
            'drivers_id'   => ['name' => 'drivers_id', 'data' => 'drivers_id', 'title' => 'Водитель'],
            'objects_id'   => ['name' => 'objects_id', 'data' => 'objects_id', 'title' => 'Объект'],
            'trailer'      => ['name' => 'trailer', 'data' => 'trailer', 'title' => 'Прицеп'],
        ];

        if (\Auth::user()->isAdmin()) {
            $array['users_id'] = ['name' => 'users_id', 'data' => 'users_id', 'title' => 'Владелец'];
        }

        return $array;
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'waybills';
    }
}
