<?php

namespace App\Http\Controllers;

use App\DataTables\CronsDataTable;
use App\Http\Requests\CreateCronsRequest;
use App\Http\Requests\UpdateCronsRequest;
use App\Repositories\CronsObjectsRepository;
use App\Repositories\CronsRepository;
use Flash;

class CronsController extends AppBaseController
{
    /** @var  CronsRepository */
    private $cronsRepository;

    public function __construct(CronsRepository $cronsRepo)
    {
        $this->cronsRepository = $cronsRepo;
    }

    /**
     * Display a listing of the Crons.
     *
     * @param CronsDataTable $cronsDataTable
     * @return Response
     */
    public function index(CronsDataTable $cronsDataTable)
    {
        return $cronsDataTable->render('crons.index');
    }

    /**
     * Show the form for creating a new Crons.
     *
     * @return Response
     */
    public function create()
    {
        return view('crons.create');
    }

    /**
     * Store a newly created Crons in storage.
     *
     * @param CreateCronsRequest $request
     * @param CronsObjectsRepository $cronsObjectsRepository
     *
     * @return Response
     */
    public function store(CreateCronsRequest $request, CronsObjectsRepository $cronsObjectsRepository)
    {
        $array = $request->except('_token', 'objects');

        if ((int)$array['owners_id'] === 0) {
            unset($array['owners_id']);
        }

        if ((int)$array['drivers_id'] === 0) {
            unset($array['drivers_id']);
        }

        if ((int)$array['trailer'] === 0) {
            unset($array['trailer']);
        }

        $crons = $this->cronsRepository->create($array);

        foreach ((array)$request->get('objects') as $item) {
            $cronsObjectsRepository->create([
                'objects_id' => (int)$item,
                'crons_id'   => $crons->id,
            ]);
        }

        Flash::success('Задание автоматизации успешно добавлено.');

        return redirect(route('crons.index'));
    }

    /**
     * Display the specified Crons.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $crons = $this->cronsRepository->findWithoutFail($id);

        if (empty($crons)) {
            Flash::error('Задание автоматизации не найдено');

            return redirect(route('crons.index'));
        }

        return view('crons.show')->with('crons', $crons);
    }

    /**
     * Show the form for editing the specified Crons.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $crons = $this->cronsRepository->findWithoutFail($id);

        if (empty($crons)) {
            Flash::error('Задание автоматизации не найдено');

            return redirect(route('crons.index'));
        }

        return view('crons.edit')->with('crons', $crons);
    }

    /**
     * Update the specified Crons in storage.
     *
     * @param  int $id
     * @param UpdateCronsRequest $request
     * @param CronsObjectsRepository $cronsObjectsRepository
     *
     * @return Response
     */
    public function update($id, UpdateCronsRequest $request, CronsObjectsRepository $cronsObjectsRepository)
    {
        $crons = $this->cronsRepository->findWithoutFail($id);

        if (empty($crons)) {
            Flash::error('Задание автоматизации не найдено');

            return redirect(route('crons.index'));
        }

        $cronsObjectsRepository->deleteWhere(['crons_id' => $crons->id]);

        foreach ((array)$request->get('objects') as $item) {
            $cronsObjectsRepository->create([
                'objects_id' => (int)$item,
                'crons_id'   => $crons->id,
            ]);
        }

        $array = $request->except('_token', 'objects');

        if ((int)$array['owners_id'] === 0) {
            unset($array['owners_id']);
        }

        if ((int)$array['drivers_id'] === 0) {
            unset($array['drivers_id']);
        }

        if ((int)$array['trailer'] === 0) {
            unset($array['trailer']);
        }

        $this->cronsRepository->update($array, $id);

        Flash::success('Задание автоматизации отредактировано.');

        return redirect(route('crons.index'));
    }

    /**
     * Remove the specified Crons from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $crons = $this->cronsRepository->findWithoutFail($id);

        if (empty($crons)) {
            Flash::error('Задание автоматизации не найдено');

            return redirect(route('crons.index'));
        }

        $this->cronsRepository->delete($id);

        Flash::success('Задание автоматизации удалено.');

        return redirect(route('crons.index'));
    }
}