<?php

namespace App\Http\Controllers;

use App\DataTables\ObjectsDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateObjectsRequest;
use App\Http\Requests\UpdateObjectsRequest;
use App\Repositories\ObjectsRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;

class ObjectsController extends AppBaseController
{
    /** @var  ObjectsRepository */
    private $objectsRepository;

    public function __construct(ObjectsRepository $objectsRepo)
    {
        $this->objectsRepository = $objectsRepo;
    }

    /**
     * Display a listing of the Objects.
     *
     * @param ObjectsDataTable $objectsDataTable
     * @return Response
     */
    public function index(ObjectsDataTable $objectsDataTable)
    {
        return $objectsDataTable->render('objects.index');
    }

    /**
     * Show the form for creating a new Objects.
     *
     * @return Response
     */
    public function create()
    {
        return view('objects.create');
    }

    /**
     * Store a newly created Objects in storage.
     *
     * @param CreateObjectsRequest $request
     *
     * @return Response
     */
    public function store(CreateObjectsRequest $request)
    {
        $input = $request->all();

        $objects = $this->objectsRepository->create($input);

        Flash::success('Объект успешно добавлен.');

        return redirect(route('objects.index'));
    }

    /**
     * Display the specified Objects.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $objects = $this->objectsRepository->findWithoutFail($id);

        if (empty($objects)) {
            Flash::error('Объект не найден');

            return redirect(route('objects.index'));
        }

        return view('objects.show')->with('objects', $objects);
    }

    /**
     * Show the form for editing the specified Objects.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $objects = $this->objectsRepository->findWithoutFail($id);

        if (empty($objects)) {
            Flash::error('Объект не найден');

            return redirect(route('objects.index'));
        }

        return view('objects.edit')->with('objects', $objects);
    }

    /**
     * Update the specified Objects in storage.
     *
     * @param  int              $id
     * @param UpdateObjectsRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateObjectsRequest $request)
    {
        $objects = $this->objectsRepository->findWithoutFail($id);

        if (empty($objects)) {
            Flash::error('Объект не найден');

            return redirect(route('objects.index'));
        }

        $objects = $this->objectsRepository->update($request->all(), $id);

        Flash::success('Объект успешно сохранен.');

        return redirect(route('objects.index'));
    }

    /**
     * Remove the specified Objects from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $objects = $this->objectsRepository->findWithoutFail($id);

        if (empty($objects)) {
            Flash::error('Объект не найден');

            return redirect(route('objects.index'));
        }

        $this->objectsRepository->delete($id);

        Flash::success('Объект успешно удален.');

        return redirect(route('objects.index'));
    }
}
