<?php

namespace App\Http\Controllers;

use App\DataTables\OwnersDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateOwnersRequest;
use App\Http\Requests\UpdateOwnersRequest;
use App\Repositories\OwnersRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;

class OwnersController extends AppBaseController
{
    /** @var  OwnersRepository */
    private $ownersRepository;

    public function __construct(OwnersRepository $ownersRepo)
    {
        $this->ownersRepository = $ownersRepo;
    }

    /**
     * Display a listing of the Owners.
     *
     * @param OwnersDataTable $ownersDataTable
     * @return Response
     */
    public function index(OwnersDataTable $ownersDataTable)
    {
        return $ownersDataTable->render('owners.index');
    }

    /**
     * Show the form for creating a new Owners.
     *
     * @return Response
     */
    public function create()
    {
        return view('owners.create');
    }

    /**
     * Store a newly created Owners in storage.
     *
     * @param CreateOwnersRequest $request
     *
     * @return Response
     */
    public function store(CreateOwnersRequest $request)
    {
        $input = $request->all();

        $owners = $this->ownersRepository->create($input);

        Flash::success('Владелец успешно добавлен.');

        return redirect(route('owners.index'));
    }

    /**
     * Display the specified Owners.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $owners = $this->ownersRepository->findWithoutFail($id);

        if (empty($owners)) {
            Flash::error('Владелец не найден');

            return redirect(route('owners.index'));
        }

        return view('owners.show')->with('owners', $owners);
    }

    /**
     * Show the form for editing the specified Owners.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $owners = $this->ownersRepository->findWithoutFail($id);

        if (empty($owners)) {
            Flash::error('Владелец не найден');

            return redirect(route('owners.index'));
        }

        return view('owners.edit')->with('owners', $owners);
    }

    /**
     * Update the specified Owners in storage.
     *
     * @param  int              $id
     * @param UpdateOwnersRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateOwnersRequest $request)
    {
        $owners = $this->ownersRepository->findWithoutFail($id);

        if (empty($owners)) {
            Flash::error('Владелец не найден');

            return redirect(route('owners.index'));
        }

        $owners = $this->ownersRepository->update($request->all(), $id);

        Flash::success('Владелец успешно отредактирован.');

        return redirect(route('owners.index'));
    }

    /**
     * Remove the specified Owners from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $owners = $this->ownersRepository->findWithoutFail($id);

        if (empty($owners)) {
            Flash::error('Владелец не найден');

            return redirect(route('owners.index'));
        }

        $this->ownersRepository->delete($id);

        Flash::success('Владелец успешно удален.');

        return redirect(route('owners.index'));
    }
}
