<?php

namespace App\Http\Controllers;

use App\DataTables\ServersDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateServersRequest;
use App\Http\Requests\UpdateServersRequest;
use App\Repositories\ServersRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;

class ServersController extends AppBaseController
{
    /** @var  ServersRepository */
    private $serversRepository;

    public function __construct(ServersRepository $serversRepo)
    {
        $this->serversRepository = $serversRepo;
    }

    /**
     * Display a listing of the Servers.
     *
     * @param ServersDataTable $serversDataTable
     * @return Response
     */
    public function index(ServersDataTable $serversDataTable)
    {
        return $serversDataTable->render('servers.index');
    }

    /**
     * Show the form for creating a new Servers.
     *
     * @return Response
     */
    public function create()
    {
        return view('servers.create');
    }

    /**
     * Store a newly created Servers in storage.
     *
     * @param CreateServersRequest $request
     *
     * @return Response
     */
    public function store(CreateServersRequest $request)
    {
        $input = $request->all();

        $servers = $this->serversRepository->create($input);

        Flash::success('Сервер успешно добавлен.');

        return redirect(route('servers.index'));
    }

    /**
     * Display the specified Servers.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $servers = $this->serversRepository->findWithoutFail($id);

        if (empty($servers)) {
            Flash::error('Сервер не найден');

            return redirect(route('servers.index'));
        }

        return view('servers.show')->with('servers', $servers);
    }

    /**
     * Show the form for editing the specified Servers.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $servers = $this->serversRepository->findWithoutFail($id);

        if (empty($servers)) {
            Flash::error('Сервер не найден');

            return redirect(route('servers.index'));
        }

        return view('servers.edit')->with('servers', $servers);
    }

    /**
     * Update the specified Servers in storage.
     *
     * @param  int              $id
     * @param UpdateServersRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateServersRequest $request)
    {
        $servers = $this->serversRepository->findWithoutFail($id);

        if (empty($servers)) {
            Flash::error('Сервер не найден');

            return redirect(route('servers.index'));
        }

        $servers = $this->serversRepository->update($request->all(), $id);

        Flash::success('Сервер успешно сохранен.');

        return redirect(route('servers.index'));
    }

    /**
     * Remove the specified Servers from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $servers = $this->serversRepository->findWithoutFail($id);

        if (empty($servers)) {
            Flash::error('Сервер не найден');

            return redirect(route('servers.index'));
        }

        $this->serversRepository->delete($id);

        Flash::success('Сервер успешно удален.');

        return redirect(route('servers.index'));
    }
}
