<?php

namespace App\Models;

use Collective\Html\Eloquent\FormAccessible;
use Eloquent as Model;

/**
 * @SWG\Definition(
 *      definition="Drivers",
 *      required={""},
 *      @SWG\Property(
 *          property="id",
 *          description="id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="active",
 *          description="active",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="users_id",
 *          description="users_id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="type",
 *          description="type",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="email",
 *          description="email",
 *          type="string"
 *      )
 * )
 */
class Crons extends Model
{
    use FormAccessible;

    public $table = 'crons';

    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';


    public $fillable = [
        'active',
        'users_id',
        'type',
        'email',
        'owners_id',
        'drivers_id',
        'trailer',
        'customer',
        'place_work',
        'type_work',
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id'         => 'integer',
        'active'     => 'boolean',
        'users_id'   => 'integer',
        'owners_id'  => 'integer',
        'drivers_id' => 'integer',
        'trailer'    => 'integer',
        'type'       => 'string',
        'email'      => 'string',
        'customer'   => 'string',
        'place_work' => 'string',
        'type_work'  => 'string',
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'users_id'   => 'required|integer',
        'type'       => 'required|string',
        'email'      => 'required|string',
        'objects'    => 'required|array',
        'owners_id'  => 'sometimes|integer',
        'drivers_id' => 'sometimes|integer',
        'trailer'    => 'sometimes|integer',
        'active'     => 'sometimes|nullable|integer',
        'customer'   => 'sometimes|nullable|string',
        'place_work' => 'sometimes|nullable|string',
        'type_work'  => 'sometimes|nullable|string',
    ];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function user()
    {
        return $this->belongsTo(\App\Models\Users::class, 'users_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     **/
    public function objects()
    {
        return $this->hasMany(\App\Models\CronsObjects::class, 'crons_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function waybills()
    {
        return $this->hasMany(\App\Models\CronsObjects::class, 'crons_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function driver()
    {
        return $this->belongsTo(\App\Models\Drivers::class, 'drivers_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function trailerF()
    {
        return $this->belongsTo(\App\Models\Objects::class, 'trailer');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function owner()
    {
        return $this->belongsTo(\App\Models\Owners::class, 'owners_id');
    }

    public function formObjectsAttribute($value)
    {
        $objects = [];

        foreach ($this->objects as $item) {
            $objects[] = $item->objects_id;
        }

        return $objects;
    }
}
