<?php

namespace App\Models;

use Eloquent as Model;

/**
 * @SWG\Definition(
 *      definition="Owners",
 *      required={""},
 *      @SWG\Property(
 *          property="id",
 *          description="id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="type",
 *          description="type",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="company",
 *          description="company",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="surname",
 *          description="surname",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="name",
 *          description="name",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="middlename",
 *          description="middlename",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="address",
 *          description="address",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="phone",
 *          description="phone",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="users_id",
 *          description="users_id",
 *          type="integer",
 *          format="int32"
 *      )
 * )
 */
class Owners extends Model
{

    public $table = 'owners';

    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';


    public $fillable = [
        'type',
        'company',
        'surname',
        'name',
        'middlename',
        'address',
        'phone',
        'users_id',
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id'         => 'integer',
        'type'       => 'string',
        'company'    => 'string',
        'surname'    => 'string',
        'name'       => 'string',
        'middlename' => 'string',
        'address'    => 'string',
        'phone'      => 'string',
        'users_id'   => 'integer',
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'type'       => 'required|in:company,individual',
        'company'    => 'required_if:type,company|nullable|string',
        'surname'    => 'sometimes|nullable|string',
        'name'       => 'required_if:type,individual|nullable|string',
        'middlename' => 'sometimes|nullable|string',
        'address'    => 'sometimes|nullable|string',
        'phone'      => 'sometimes|nullable|string',
        'users_id'   => 'required|integer',
    ];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function user()
    {
        return $this->belongsTo(\App\Models\Users::class, 'users_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function waybills()
    {
        return $this->hasMany(\App\Models\Waybills::class, 'owners_id', 'id');
    }

    public function getFriendlyTypeAttribute()
    {
        if ($this->type === 'company') {
            return 'Компания';
        }

        return 'Частное лицо';
    }
}
