<?php

namespace App\Models;

use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User;

/**
 * @SWG\Definition(
 *      definition="Users",
 *      required={""},
 *      @SWG\Property(
 *          property="id",
 *          description="id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="username",
 *          description="username",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="password",
 *          description="password",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="mon_id",
 *          description="mon_id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="servers_id",
 *          description="servers_id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="remember_token",
 *          description="remember_token",
 *          type="string"
 *      )
 * )
 */
class Users extends User
{
    use Notifiable;

    public $table = 'users';

    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';


    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    public $fillable = [
        'username',
        'password',
        'danger_password',
        'admin',
        'mon_id',
        'servers_id',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id'              => 'integer',
        'username'        => 'string',
        'password'        => 'string',
        'danger_password' => 'string',
        'admin'           => 'integer',
        'mon_id'          => 'integer',
        'servers_id'      => 'integer',
        'remember_token'  => 'string',
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'username'   => 'required|string',
        'password'   => 'required|confirmed|string',
        'admin'      => 'sometimes|nullable|integer',
        'mon_id'     => 'sometimes|nullable|integer',
        'servers_id' => 'sometimes|nullable|integer',
    ];

    public function isAdmin()
    {
        return $this->admin === 1;
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function server()
    {
        return $this->belongsTo(\App\Models\Servers::class, 'servers_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function drivers()
    {
        return $this->hasMany(\App\Models\Drivers::class, 'users_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function objects()
    {
        return $this->hasMany(\App\Models\Objects::class, 'users_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function owners()
    {
        return $this->hasMany(\App\Models\Owners::class, 'users_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function waybills()
    {
        return $this->hasMany(\App\Models\Waybills::class, 'users_id', 'id');
    }
}
