<?php

namespace App\Models;

use Eloquent as Model;

/**
 * @SWG\Definition(
 *      definition="Waybills",
 *      required={""},
 *      @SWG\Property(
 *          property="id",
 *          description="id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="fuel_top",
 *          description="fuel_top",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="fuel_return",
 *          description="fuel_return",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="tucked",
 *          description="tucked",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="spent",
 *          description="spent",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="odometer",
 *          description="odometer",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="mileage",
 *          description="mileage",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="trailer",
 *          description="trailer",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="users_id",
 *          description="users_id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="owners_id",
 *          description="owners_id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="drivers_id",
 *          description="drivers_id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="objects_id",
 *          description="objects_id",
 *          type="integer",
 *          format="int32"
 *      )
 * )
 */
class Waybills extends Model
{

    public $table = 'waybills';

    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';

    public $fillable = [
        'start_date',
        'end_date',
        'out_date',
        'arrival_date',
        'fuel_top',
        'fuel_return',
        'tucked',
        'spent',
        'odometer',
        'mileage',
        'trailer',
        'users_id',
        'owners_id',
        'drivers_id',
        'objects_id',
        'file',
        'type',
        'customer',
        'place_work',
        'type_work',
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id'           => 'integer',
        'fuel_top'     => 'string',
        'fuel_return'  => 'string',
        'tucked'       => 'string',
        'spent'        => 'string',
        'odometer'     => 'string',
        'mileage'      => 'string',
        'trailer'      => 'integer',
        'users_id'     => 'integer',
        'owners_id'    => 'integer',
        'drivers_id'   => 'integer',
        'objects_id'   => 'integer',
        'start_date'   => 'datetime',
        'end_date'     => 'datetime',
        'out_date'     => 'datetime',
        'arrival_date' => 'datetime',
        'file'         => 'string',
        'type'         => 'string',
        'customer'     => 'string',
        'place_work'   => 'string',
        'type_work'    => 'string',
    ];

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = [
        'created_at',
        'updated_at',
        'start_date',
        'end_date',
        'out_date',
        'arrival_date',
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'start_date' => 'required|date',
        'end_date'   => 'required|date',
        'users_id'   => 'required|integer',
        'objects_id' => 'required|integer',
        'odometer'   => 'sometimes|nullable|string',
        'file'       => 'sometimes|string',
        'type'       => 'sometimes|string',
        'customer'   => 'sometimes|string',
        'place_work' => 'sometimes|string',
        'type_work'  => 'sometimes|string',
        'owners_id'  => 'sometimes|integer',
        'drivers_id' => 'sometimes|integer',
        'trailer'    => 'sometimes|integer',
    ];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function driver()
    {
        return $this->belongsTo(\App\Models\Drivers::class, 'drivers_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function object()
    {
        return $this->belongsTo(\App\Models\Objects::class, 'objects_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function trailerF()
    {
        return $this->belongsTo(\App\Models\Objects::class, 'trailer');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function owner()
    {
        return $this->belongsTo(\App\Models\Owners::class, 'owners_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function user()
    {
        return $this->belongsTo(\App\Models\Users::class, 'users_id');
    }
}
