<?php

use morphos\Russian\CardinalNumeralGenerator;
use morphos\Russian\NounPluralization;

if (!function_exists('getOwners')) {

    /**
     * description
     *
     * @param
     * @return array
     */
    function getOwners($user)
    {
        $result = ['0' => 'Нет'];
        $owners = \App\Models\Owners::where('users_id', $user)->get();

        foreach ($owners as $owner) {
            if ($owner->type === 'company') {
                $result[$owner->id] = $owner->company;
                continue;
            }

            $result[$owner->id] = implode(' ', [$owner->surname, $owner->name, $owner->middlename]);
        }

        return $result;
    }
}

if (!function_exists('getDrivers')) {

    /**
     * description
     *
     * @param
     * @return array
     */
    function getDrivers($user)
    {
        $result = ['0' => 'Нет'];
        $drivers = \App\Models\Drivers::where('users_id', $user)->get();

        foreach ($drivers as $driver) {
            $result[$driver->id] = implode(' ', [$driver->surname, $driver->name, $driver->middlename]);
        }

        return $result;
    }
}

if (!function_exists('getObjects')) {

    /**
     * description
     *
     * @param int $user
     * @param bool $non
     * @return array
     */
    function getObjects($user, $non = true)
    {
        $result = [];

        if ($non) {
            $result['0'] = 'Нет';
        }

        $objects = \App\Models\Objects::where('users_id', $user)->where('type', '!=', 'trailer')->get();

        foreach ($objects as $object) {
            $result[$object->id] = $object->name;
        }

        return $result;
    }
}

if (!function_exists('getTrailers')) {

    /**
     * description
     *
     * @param
     * @return array
     */
    function getTrailers($user)
    {
        $result = ['0' => 'Нет'];
        $objects = \App\Models\Objects::where('users_id', $user)->where('type', 'trailer')->get();

        foreach ($objects as $object) {
            $result[$object->id] = $object->name;
        }

        return $result;
    }
}

if (!function_exists('getRussianNameMonth')) {

    /**
     * description
     *
     * @param string|integer $month
     * @param string $morph
     * @return string
     */
    function getRussianNameMonth($month, $morph = 'ь')
    {
        $result = [
            '1'  => 'Январ',
            '2'  => 'Феврал',
            '3'  => 'Март',
            '4'  => 'Апрел',
            '5'  => 'Ма',
            '6'  => 'Июн',
            '7'  => 'Июл',
            '8'  => 'Август',
            '9'  => 'Сентябр',
            '10' => 'Октябр',
            '11' => 'Ноябр',
            '12' => 'Декабр',
        ];

        if ($morph === 'я' && ((int)$month === 3 || (int)$month === 8)) {
            $morph = 'а';
        }

        if ($morph === 'ь') {
            if ((int) $morph === 5) {
                $morph = 'й';
            } elseif ((int)$month === 3 || (int)$month === 8) {
                $morph = '';
            }
        }

        return $result[$month] . $morph;
    }
}

if (!function_exists('getTypesWaybill')) {

    /**
     * description
     *
     * @param string $key
     * @return string|array
     */
    function getTypesWaybill($key = '')
    {
        $array = [
            '4s' => 'Путевой лист - Форма №4С',
            'f1' => 'Путевой лист - Форма №1',
            'esm3' => 'ЭСМ - 3',
            'esm4' => 'ЭСМ - 4',
            'esm7' => 'ЭСМ - 7',
        ];

        if (!empty($key)) {
            return $array[$key];
        }

        return $array;
    }
}

if (!function_exists('machineHoursPluralize')) {

    /**
     * description
     *
     * @param string $machine_hours
     * @return string
     * @internal param string $key
     */
    function machineHoursPluralize($machine_hours = '0:00:00')
    {
        list($hours, $minutes, $seconds) = array_pad(explode(':', $machine_hours, 3), 3, 0);

        $hour = CardinalNumeralGenerator::getCase($hours, 'именительный');
        $hour_p = NounPluralization::pluralize('час', $hours);

        $minute = CardinalNumeralGenerator::getCase($minutes, 'именительный');
        $minute_p = NounPluralization::pluralize('минута', $minutes);

        $second = CardinalNumeralGenerator::getCase($seconds, 'именительный');
        $second_p = NounPluralization::pluralize('секунда', $seconds);

        return implode(' ', [$hour, $hour_p, $minute, $minute_p, $second, $second_p]);
    }
}