<?php

use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class CreateOwnersTable extends Migration
{
    /**
     * Schema table name to migrate
     * @var string
     */
    public $set_schema_table = 'owners';

    /**
     * Run the migrations.
     * @table owners
     *
     * @return void
     */
    public function up()
    {
        Schema::create($this->set_schema_table, function (Blueprint $table) {
            $table->engine = 'InnoDB';
            $table->increments('id');
            $table->enum('type', ['company', 'individual'])->default('individual');
            $table->text('company')->nullable();
            $table->string('surname', 45)->nullable();
            $table->string('name', 45)->nullable();
            $table->string('middlename', 45)->nullable();
            $table->string('address', 45)->nullable();
            $table->string('phone', 45)->nullable();
            $table->unsignedInteger('users_id')->nullable();

            $table->index(["users_id"], 'fk_owners_users1_idx');
            $table->nullableTimestamps();


            $table->foreign('users_id', 'fk_owners_users1_idx')
                ->references('id')->on('users')
                ->onDelete('no action')
                ->onUpdate('no action');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
     public function down()
     {
       Schema::dropIfExists($this->set_schema_table);
     }
}
