<?php

use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class CreateObjectsTable extends Migration
{
    /**
     * Schema table name to migrate
     * @var string
     */
    public $set_schema_table = 'objects';

    /**
     * Run the migrations.
     * @table objects
     *
     * @return void
     */
    public function up()
    {
        Schema::create($this->set_schema_table, function (Blueprint $table) {
            $table->engine = 'InnoDB';
            $table->increments('id');
            $table->text('name')->nullable();
            $table->text('gos_number')->nullable();
            $table->text('garage_number')->nullable();
            $table->text('model')->nullable();
            $table->string('fuel')->nullable();
            $table->enum('type', ['car', 'truck', 'tractor', 'trailer'])->default('car');
            $table->integer('mon_id')->nullable();
            $table->unsignedInteger('users_id')->nullable();
            $table->unsignedInteger('servers_id')->nullable();

            $table->index(["servers_id"], 'fk_objects_servers1_idx');

            $table->index(["users_id"], 'fk_objects_users1_idx');
            $table->nullableTimestamps();


            $table->foreign('users_id', 'fk_objects_users1_idx')
                ->references('id')->on('users')
                ->onDelete('no action')
                ->onUpdate('no action');

            $table->foreign('servers_id', 'fk_objects_servers1_idx')
                ->references('id')->on('servers')
                ->onDelete('no action')
                ->onUpdate('no action');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
     public function down()
     {
       Schema::dropIfExists($this->set_schema_table);
     }
}
