<?php

use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class CreateWaybillsTable extends Migration
{
    /**
     * Schema table name to migrate
     * @var string
     */
    public $set_schema_table = 'waybills';

    /**
     * Run the migrations.
     * @table waybills
     *
     * @return void
     */
    public function up()
    {
        Schema::create($this->set_schema_table, function (Blueprint $table) {
            $table->engine = 'InnoDB';
            $table->increments('id');
            $table->dateTime('start_date')->nullable();
            $table->dateTime('end_date')->nullable();
            $table->dateTime('out_date')->nullable();
            $table->dateTime('arrival_date')->nullable();
            $table->string('fuel_top', 45)->nullable();
            $table->string('fuel_return', 45)->nullable();
            $table->string('tucked', 45)->nullable();
            $table->string('spent', 45)->nullable();
            $table->string('odometer', 45)->nullable();
            $table->string('mileage', 45)->nullable();
            $table->string('file')->nullable();
            $table->integer('trailer')->nullable();
            $table->unsignedInteger('users_id')->nullable();
            $table->unsignedInteger('owners_id')->nullable();
            $table->unsignedInteger('drivers_id')->nullable();
            $table->unsignedInteger('objects_id')->nullable();

            $table->index(["objects_id"], 'fk_waybills_objects1_idx');

            $table->index(["owners_id"], 'fk_waybills_owners1_idx');

            $table->index(["drivers_id"], 'fk_waybills_drivers1_idx');

            $table->index(["users_id"], 'fk_waybills_users1_idx');
            $table->nullableTimestamps();


            $table->foreign('users_id', 'fk_waybills_users1_idx')
                ->references('id')->on('users')
                ->onDelete('no action')
                ->onUpdate('no action');

            $table->foreign('owners_id', 'fk_waybills_owners1_idx')
                ->references('id')->on('owners')
                ->onDelete('no action')
                ->onUpdate('no action');

            $table->foreign('drivers_id', 'fk_waybills_drivers1_idx')
                ->references('id')->on('drivers')
                ->onDelete('no action')
                ->onUpdate('no action');

            $table->foreign('objects_id', 'fk_waybills_objects1_idx')
                ->references('id')->on('objects')
                ->onDelete('no action')
                ->onUpdate('no action');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
     public function down()
     {
       Schema::dropIfExists($this->set_schema_table);
     }
}
