<?php

use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class CreateCronsTable extends Migration
{
    /**
     * Schema table name to migrate
     * @var string
     */
    public $set_schema_table = 'crons';

    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create($this->set_schema_table, function (Blueprint $table)
        {
            $table->engine = 'InnoDB';
            $table->increments('id');
            $table->integer('active')->default(0);
            $table->unsignedInteger('users_id')->nullable();
            $table->string('type')->default('f4');
            $table->string('email')->nullable();
            $table->string('customer')->nullable();
            $table->string('place_work')->nullable();
            $table->string('type_work')->nullable();
            $table->unsignedInteger('owners_id')->nullable();
            $table->unsignedInteger('drivers_id')->nullable();
            $table->unsignedInteger('objects_id')->nullable();
            $table->unsignedInteger('trailer')->nullable();

            $table->index(["trailer"], 'fk_crons_objects2_idx');

            $table->index(["owners_id"], 'fk_crons_owners1_idx');

            $table->index(["drivers_id"], 'fk_crons_drivers1_idx');

            $table->index(['users_id'], 'fk_crons_users1_idx');
            $table->nullableTimestamps();

            $table->foreign('users_id', 'fk_crons_users1_idx')
                ->references('id')->on('users')
                ->onDelete('no action')
                ->onUpdate('no action');

            $table->foreign('owners_id', 'fk_crons_owners1_idx')
                ->references('id')->on('owners')
                ->onDelete('no action')
                ->onUpdate('no action');

            $table->foreign('drivers_id', 'fk_crons_drivers1_idx')
                ->references('id')->on('drivers')
                ->onDelete('no action')
                ->onUpdate('no action');

            $table->foreign('trailer', 'fk_crons_objects2_idx')
                ->references('id')->on('objects')
                ->onDelete('no action')
                ->onUpdate('no action');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists($this->set_schema_table);
    }
}
