<?php

namespace Monitorit\Installer\Helpers;

use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\File;

class EnvironmentManager
{
    /**
     * @var string
     */
    private $envPath;

    /**
     * @var string
     */
    private $envExamplePath;

    /**
     * Set the .env and .env.example paths.
     */
    public function __construct()
    {
        $this->envPath        = base_path('.env');
        $this->envExamplePath = base_path('.env.example');
    }

    /**
     * Get the content of the .env file.
     *
     * @return string
     */
    public function getEnvContent()
    {
        if ( ! File::exists($this->envPath)) {
            if (File::exists($this->envExamplePath)) {
                copy($this->envExamplePath, $this->envPath);
            } else {
                touch($this->envPath);
            }
        }

        return File::get($this->envPath);
    }

    /**
     * Get the config of the .env file.
     *
     * @return array
     */
    public function getEnvConfig()
    {
        return [
            'db_host'   => env('DB_HOST', ''),
            'db_port'   => env('DB_PORT', ''),
            'db_name'   => env('DB_DATABASE', ''),
            'db_user'   => env('DB_USERNAME', ''),
            'db_pass'   => env('DB_PASSWORD', ''),
            'mail_host' => env('MAIL_HOST', ''),
            'mail_port' => env('MAIL_PORT', ''),
            'mail_enc'  => env('MAIL_ENCRYPTION', ''),
            'mail_user' => env('MAIL_USERNAME', ''),
            'mail_pass' => env('MAIL_PASSWORD', ''),
        ];
    }

    /**
     * Save the edited content to the file.
     *
     * @param Request $request
     *
     * @return string
     */
    public function saveFile(Request $request)
    {
        $message = trans('installer_messages.environment.success');

        $env = $this->getEnvContent();

        $dbHost   = $request->get('db_host', '');
        $dbPort   = $request->get('db_port', '');
        $dbName   = $request->get('db_name', '');
        $dbUser   = $request->get('db_user', '');
        $dbPass   = $request->get('db_pass', '');
        $mailHost = $request->get('mail_host', '');
        $mailPort = $request->get('mail_port', '');
        $mailEnc  = $request->get('mail_enc', '');
        $mailUser = $request->get('mail_user', '');
        $mailPass = $request->get('mail_pass', '');

        $databaseSetting = 'DB_HOST=' . $dbHost . '
DB_PORT=' . $dbPort . '
DB_DATABASE=' . $dbName . '
DB_USERNAME=' . $dbUser . '
DB_PASSWORD=' . $dbPass . '
';

        $mailSetting = 'MAIL_HOST=' . $mailHost . '
MAIL_PORT=' . $mailPort . '
MAIL_ENCRYPTION=' . $mailEnc . '
MAIL_USERNAME=' . $mailUser . '
MAIL_PASSWORD=' . $mailPass . '
';

        $rows        = explode("\n", $env);
        $unwanted    = 'DB_HOST|DB_PORT|DB_DATABASE|DB_USERNAME|DB_PASSWORD|MAIL_HOST|MAIL_PORT|MAIL_ENCRYPTION|MAIL_USERNAME|MAIL_PASSWORD';
        $cleanArray  = preg_grep("/$unwanted/i", $rows, PREG_GREP_INVERT);
        $cleanString = implode("\n", $cleanArray);
        $env         = $cleanString . $databaseSetting . $mailSetting;

        try {
            File::put($this->envPath, $env);
        } catch (Exception $e) {
            $message = trans('installer_messages.environment.errors');
        }

        return $message;
    }
}