<?php

namespace Monitorit\Installer\Providers;

use Illuminate\Routing\Router;
use Illuminate\Support\ServiceProvider;
use Monitorit\Installer\Middleware\canInstall;
use Monitorit\Installer\Middleware\canUpdate;
use Monitorit\Installer\Middleware\RedirectInstall;

class InstallerServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->publishFiles();

        if (version_compare($this->laravelVersion(), '5.4.0', '>=')) {
            $this->loadRoutesFrom(__DIR__ . '/../routes.php');
        } else {
            include __DIR__ . '/../routes.php';
        }
    }

    /**
     * Bootstrap the application events.
     *
     * @param Router $router
     */
    public function boot(Router $router)
    {
        $router->pushMiddlewareToGroup('web', RedirectInstall::class);

        $router->middlewareGroup('install', [
            canInstall::class,
        ]);
    }

    /**
     * Publish config file for the installer.
     *
     * @return void
     */
    protected function publishFiles()
    {
        $this->publishes([
            __DIR__ . '/../config/installer.php' => base_path('config/installer.php'),
        ]);

        $this->publishes([
            __DIR__ . '/../assets' => public_path('installer'),
        ], 'public');

        $this->publishes([
            __DIR__ . '/../views' => base_path('resources/views/vendor/installer'),
        ]);

        $this->publishes([
            __DIR__ . '/../lang' => base_path('resources/lang'),
        ]);
    }

    protected function laravelVersion()
    {
        $laravel = app();
        return $laravel::VERSION;
    }
}
