<?php

use App\Models\Objects;
use App\Repositories\ObjectsRepository;
use Illuminate\Foundation\Testing\DatabaseTransactions;

class ObjectsRepositoryTest extends TestCase
{
    use MakeObjectsTrait, ApiTestTrait, DatabaseTransactions;

    /**
     * @var ObjectsRepository
     */
    protected $objectsRepo;

    public function setUp()
    {
        parent::setUp();
        $this->objectsRepo = App::make(ObjectsRepository::class);
    }

    /**
     * @test create
     */
    public function testCreateObjects()
    {
        $objects = $this->fakeObjectsData();
        $createdObjects = $this->objectsRepo->create($objects);
        $createdObjects = $createdObjects->toArray();
        $this->assertArrayHasKey('id', $createdObjects);
        $this->assertNotNull($createdObjects['id'], 'Created Objects must have id specified');
        $this->assertNotNull(Objects::find($createdObjects['id']), 'Objects with given id must be in DB');
        $this->assertModelData($objects, $createdObjects);
    }

    /**
     * @test read
     */
    public function testReadObjects()
    {
        $objects = $this->makeObjects();
        $dbObjects = $this->objectsRepo->find($objects->id);
        $dbObjects = $dbObjects->toArray();
        $this->assertModelData($objects->toArray(), $dbObjects);
    }

    /**
     * @test update
     */
    public function testUpdateObjects()
    {
        $objects = $this->makeObjects();
        $fakeObjects = $this->fakeObjectsData();
        $updatedObjects = $this->objectsRepo->update($fakeObjects, $objects->id);
        $this->assertModelData($fakeObjects, $updatedObjects->toArray());
        $dbObjects = $this->objectsRepo->find($objects->id);
        $this->assertModelData($fakeObjects, $dbObjects->toArray());
    }

    /**
     * @test delete
     */
    public function testDeleteObjects()
    {
        $objects = $this->makeObjects();
        $resp = $this->objectsRepo->delete($objects->id);
        $this->assertTrue($resp);
        $this->assertNull(Objects::find($objects->id), 'Objects should not exist in DB');
    }
}
