<?php

use App\Models\Owners;
use App\Repositories\OwnersRepository;
use Illuminate\Foundation\Testing\DatabaseTransactions;

class OwnersRepositoryTest extends TestCase
{
    use MakeOwnersTrait, ApiTestTrait, DatabaseTransactions;

    /**
     * @var OwnersRepository
     */
    protected $ownersRepo;

    public function setUp()
    {
        parent::setUp();
        $this->ownersRepo = App::make(OwnersRepository::class);
    }

    /**
     * @test create
     */
    public function testCreateOwners()
    {
        $owners = $this->fakeOwnersData();
        $createdOwners = $this->ownersRepo->create($owners);
        $createdOwners = $createdOwners->toArray();
        $this->assertArrayHasKey('id', $createdOwners);
        $this->assertNotNull($createdOwners['id'], 'Created Owners must have id specified');
        $this->assertNotNull(Owners::find($createdOwners['id']), 'Owners with given id must be in DB');
        $this->assertModelData($owners, $createdOwners);
    }

    /**
     * @test read
     */
    public function testReadOwners()
    {
        $owners = $this->makeOwners();
        $dbOwners = $this->ownersRepo->find($owners->id);
        $dbOwners = $dbOwners->toArray();
        $this->assertModelData($owners->toArray(), $dbOwners);
    }

    /**
     * @test update
     */
    public function testUpdateOwners()
    {
        $owners = $this->makeOwners();
        $fakeOwners = $this->fakeOwnersData();
        $updatedOwners = $this->ownersRepo->update($fakeOwners, $owners->id);
        $this->assertModelData($fakeOwners, $updatedOwners->toArray());
        $dbOwners = $this->ownersRepo->find($owners->id);
        $this->assertModelData($fakeOwners, $dbOwners->toArray());
    }

    /**
     * @test delete
     */
    public function testDeleteOwners()
    {
        $owners = $this->makeOwners();
        $resp = $this->ownersRepo->delete($owners->id);
        $this->assertTrue($resp);
        $this->assertNull(Owners::find($owners->id), 'Owners should not exist in DB');
    }
}
