<?php

use App\Models\Waybills;
use App\Repositories\WaybillsRepository;
use Illuminate\Foundation\Testing\DatabaseTransactions;

class WaybillsRepositoryTest extends TestCase
{
    use MakeWaybillsTrait, ApiTestTrait, DatabaseTransactions;

    /**
     * @var WaybillsRepository
     */
    protected $waybillsRepo;

    public function setUp()
    {
        parent::setUp();
        $this->waybillsRepo = App::make(WaybillsRepository::class);
    }

    /**
     * @test create
     */
    public function testCreateWaybills()
    {
        $waybills = $this->fakeWaybillsData();
        $createdWaybills = $this->waybillsRepo->create($waybills);
        $createdWaybills = $createdWaybills->toArray();
        $this->assertArrayHasKey('id', $createdWaybills);
        $this->assertNotNull($createdWaybills['id'], 'Created Waybills must have id specified');
        $this->assertNotNull(Waybills::find($createdWaybills['id']), 'Waybills with given id must be in DB');
        $this->assertModelData($waybills, $createdWaybills);
    }

    /**
     * @test read
     */
    public function testReadWaybills()
    {
        $waybills = $this->makeWaybills();
        $dbWaybills = $this->waybillsRepo->find($waybills->id);
        $dbWaybills = $dbWaybills->toArray();
        $this->assertModelData($waybills->toArray(), $dbWaybills);
    }

    /**
     * @test update
     */
    public function testUpdateWaybills()
    {
        $waybills = $this->makeWaybills();
        $fakeWaybills = $this->fakeWaybillsData();
        $updatedWaybills = $this->waybillsRepo->update($fakeWaybills, $waybills->id);
        $this->assertModelData($fakeWaybills, $updatedWaybills->toArray());
        $dbWaybills = $this->waybillsRepo->find($waybills->id);
        $this->assertModelData($fakeWaybills, $dbWaybills->toArray());
    }

    /**
     * @test delete
     */
    public function testDeleteWaybills()
    {
        $waybills = $this->makeWaybills();
        $resp = $this->waybillsRepo->delete($waybills->id);
        $this->assertTrue($resp);
        $this->assertNull(Waybills::find($waybills->id), 'Waybills should not exist in DB');
    }
}
