<?php

use Faker\Factory as Faker;
use App\Models\Objects;
use App\Repositories\ObjectsRepository;

trait MakeObjectsTrait
{
    /**
     * Create fake instance of Objects and save it in database
     *
     * @param array $objectsFields
     * @return Objects
     */
    public function makeObjects($objectsFields = [])
    {
        /** @var ObjectsRepository $objectsRepo */
        $objectsRepo = App::make(ObjectsRepository::class);
        $theme = $this->fakeObjectsData($objectsFields);
        return $objectsRepo->create($theme);
    }

    /**
     * Get fake instance of Objects
     *
     * @param array $objectsFields
     * @return Objects
     */
    public function fakeObjects($objectsFields = [])
    {
        return new Objects($this->fakeObjectsData($objectsFields));
    }

    /**
     * Get fake data of Objects
     *
     * @param array $postFields
     * @return array
     */
    public function fakeObjectsData($objectsFields = [])
    {
        $fake = Faker::create();

        return array_merge([
            'name' => $fake->text,
            'gos_number' => $fake->text,
            'type' => $fake->word,
            'mon_id' => $fake->randomDigitNotNull,
            'users_id' => $fake->randomDigitNotNull,
            'servers_id' => $fake->randomDigitNotNull,
            'created_at' => $fake->date('Y-m-d H:i:s'),
            'updated_at' => $fake->date('Y-m-d H:i:s')
        ], $objectsFields);
    }
}
