<?php

use Faker\Factory as Faker;
use App\Models\Waybills;
use App\Repositories\WaybillsRepository;

trait MakeWaybillsTrait
{
    /**
     * Create fake instance of Waybills and save it in database
     *
     * @param array $waybillsFields
     * @return Waybills
     */
    public function makeWaybills($waybillsFields = [])
    {
        /** @var WaybillsRepository $waybillsRepo */
        $waybillsRepo = App::make(WaybillsRepository::class);
        $theme = $this->fakeWaybillsData($waybillsFields);
        return $waybillsRepo->create($theme);
    }

    /**
     * Get fake instance of Waybills
     *
     * @param array $waybillsFields
     * @return Waybills
     */
    public function fakeWaybills($waybillsFields = [])
    {
        return new Waybills($this->fakeWaybillsData($waybillsFields));
    }

    /**
     * Get fake data of Waybills
     *
     * @param array $postFields
     * @return array
     */
    public function fakeWaybillsData($waybillsFields = [])
    {
        $fake = Faker::create();

        return array_merge([
            'start_date' => $fake->date('Y-m-d H:i:s'),
            'end_date' => $fake->date('Y-m-d H:i:s'),
            'out_date' => $fake->date('Y-m-d H:i:s'),
            'arrival_date' => $fake->date('Y-m-d H:i:s'),
            'fuel_top' => $fake->word,
            'fuel_return' => $fake->word,
            'tucked' => $fake->word,
            'spent' => $fake->word,
            'odometer' => $fake->word,
            'mileage' => $fake->word,
            'trailer' => $fake->randomDigitNotNull,
            'users_id' => $fake->randomDigitNotNull,
            'owners_id' => $fake->randomDigitNotNull,
            'drivers_id' => $fake->randomDigitNotNull,
            'objects_id' => $fake->randomDigitNotNull,
            'created_at' => $fake->date('Y-m-d H:i:s'),
            'updated_at' => $fake->date('Y-m-d H:i:s')
        ], $waybillsFields);
    }
}
